#ifndef CENGINE_H
#define CENGINE_H

#include "libboxmanage_global.h"

#include <QList>

namespace Box {

typedef struct {
    QString strName;
    QString strPath;
    QString strOwner;
    QString strTime;
    int nEncrypted;     //1:加密 0：未加密
    int nMounted;       //1:未上锁 0：上锁
}SBoxItem;

class LIBBOXMANAGE_EXPORT CEngine
{
public:
    static CEngine *getInstance()
    {
        static CEngine *_instance = nullptr;

        if(!_instance) {
            _instance = new CEngine();
        }

        return _instance;
    }

    virtual ~CEngine();

    bool get_enable();

    int get_boxList(QList<SBoxItem> &boxList);
    int get_boxInfoByName(const QString &strName, SBoxItem & boxInfo);

    int create_normalBox(const QString &strName);
    int create_encryptBox(const QString &strName, const QString &strPasswd);
    int create_encryptBoxV2(const QString &strName, const QString &strPasswd);

    int delete_box(const QString &strName);
    int delete_boxV2(const QString &strName, QString passwd);
    int delete_boxV2_directly(const QString &strName);

    int change_boxName(const QString &strOldName, const QString &strNewName);
    int change_boxNameV2(const QString &strOldName, const QString &strNewName, const QString passwd);

    int change_boxPasswd(const QString &strName, const QString &strOldPasswd, const QString &strNewPasswd);
    int change_boxPasswdV2(const QString &strName, const QString &strOldPasswd, const QString &strNewPasswd);

    bool verify_boxPassphrase(const QString &strName, const QString &strPasswd);
    bool verify_boxPassphraseByBoxSM(const QString &strBoxname, const QString &newPasswd);

    bool verify_boxPassphrase_v2(const QString& strName, const QString& strPasswd, QString& error);
    bool verify_boxPassphraseByBoxSM_v2(const QString& strBoxname, const QString& newPasswd, QString& error);

    int mount_ecnryptedBox(const QString &strName, const QString strPasswd);
    int mount_ecnryptedBoxV2(const QString &strName, const QString strPasswd);
    int mount_ecnryptedBoxByBoxSM(const QString &strName, const QString strPasswd);

    int mount_encryptedBoxAllByKey(const QString& globalKey);
    int mount_ecnryptedBoxByKey(const QString &strName, const QString &key);
    int mount_ecnryptedBoxByKeyByBoxSM(const QString &strName, const QString &key);

    int umount_encryptedBox(const QString &strName);
    int umount_encryptedBoxV2(const QString &strName);
    int umount_encryptedBoxByBoxSM(const QString &strName);

    bool is_boxExist(const QString &box_name);
    int change_boxPasswdByGlobalKey(const QString &strBoxname, const QString &pswTxt, const QString &newPasswd);
    int change_boxPasswdByGlobalKeyV2(const QString &strBoxname, const QString &pswTxt, const QString &newPasswd);
    int create_GlobalKey(QString &strKey);

    bool verify_GlobalKey(const QString &key);
    bool verify_GlobalKeyByBoxSM(const QString &key);
    bool verify_GlobalKey_v2(const QString &strBoxname, const QString &key, QString &error);
    bool verify_GlobalKeyByBoxSM_v2(const QString &strBoxname, const QString &key, QString &error);
    bool box_GlobalKeyExist();

    int create_normalBoxByBoxSM(const QString &strName);
    int create_encryptBoxByBoxSM(const QString &strName, const QString &strPasswd);

    int change_boxNameByBoxSM(const QString &strOldName, const QString &strNewName);
    int change_boxNameByBoxSM(const QString &strOldName, const QString &strNewName, const QString &password);
    int change_boxPasswdByBoxSM(const QString &strName, const QString &strOldPasswd, const QString &strNewPasswd);
    int change_boxPasswdUseBoxSMByGlobalKey(const QString &strBoxname, const QString &pswTxt, const QString &newPasswd);

    int delete_boxByBoxSM(const QString &strName);
    int delete_boxByBoxSM(const QString &strName, const QString &password);
    int delete_boxDirectlyByBoxSM(const QString &strName);
    int create_GlobalKeyByBoxSM(QString &strKey);

    int get_boxListByBoxSM(QList<SBoxItem> &boxList);
    int get_boxInfoByNameByBoxSM(const QString &strName, SBoxItem &boxInfo);

    int init_box();

    QString get_currentUserName();

    int get_encryptType();
    //0 Ext4格式， <0 失败； >0 其他格式
    int get_fsType();

    int create_resetKeyHwUfs(QString boxName, QString passwd, QString keyFile);
    int create_resetKeyHwUfsByBoxSM(QString boxName, QString passwd, QString keyFile);

    bool check_cryptoCreateEnable();

    bool isCryptoMounted(const QString& boxName);

private:
    CEngine();

private:
    bool m_enable;
    bool m_enableDebug;
};

}

#endif // CENGINE_H
